from distutils.core import Command
from distutils.errors import DistutilsOptionError

from coalib.misc.Constants import Constants


class BuildDbusService(Command):
    """
    Add a `build_dbus` command  to your setup.py.
    To use this Command class add a command to call this class::

        # For setuptools
        setup(
              entry_points={
                "distutils.commands": [
                    "build_dbus = "
                    "coalib.misc.BuildDbusService:BuildDbusService"
                ]
              }
        )

        # For distutils
        from coalib.misc.BuildDbusService import BuildDbusService
        setup(
              cmdclass={'build_dbus': BuildDbusService}
        )

    You can then use the following setup command to produce a dbus service::

        $ python setup.py build_dbus
    """
    user_options = [('output=', 'O', 'output file')]

    def initialize_options(self):
        self.output = None

    def finalize_options(self):
        if self.output is None:
            raise DistutilsOptionError('\'output\' option is required')
        self.announce('Writing dbus service %s' % self.output)

    def run(self):
        dist = self.distribution
        dbus_service = ("[D-BUS Service]\n"
                        "Names=" + Constants.BUS_NAME + "\n"
                        "Exec=coala-dbus")

        with open(self.output, 'w') as f:
            f.write(dbus_service)




# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json

from coalib.output.printers.ListLogPrinter import ListLogPrinter
from coalib.output.JSONEncoder import JSONEncoder
from coalib.coala_main import run_coala


def main():
    log_printer = ListLogPrinter()

    results, exitcode = run_coala(log_printer)

    retval = {"logs": log_printer.logs, "results": results}
    print(json.dumps(retval,
                     cls=JSONEncoder,
                     sort_keys=True,
                     indent=2,
                     separators=(',', ': ')))

    return exitcode



from isort import SortImports

from coalib.bears.LocalBear import LocalBear
from coalib.results.Diff import Diff
from coalib.results.Result import Result


class PyImportSortBear(LocalBear):
    def run(self, filename, file):
        """
        Sorts imports for python.
        """
        new_file = SortImports(
            file_contents=''.join(file)).output.splitlines(True)
        if new_file != file:
            diff = Diff.from_string_arrays(file, new_file)
            yield Result(self,
                         "Imports can be sorted.",
                         affected_code=diff.affected_code(filename),
                         diffs={filename: diff})



# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import functools
from pyprint.ConsolePrinter import ConsolePrinter

from coalib.output.ConsoleInteraction import (nothing_done,
                                              acquire_settings,
                                              print_section_beginning,
                                              print_results,
                                              show_bears)
from coalib.coala_main import run_coala


def main():
    console_printer = ConsolePrinter()
    partial_show_bears = functools.partial(
        show_bears,
        console_printer=console_printer)
    partial_print_sec_beg = functools.partial(
        print_section_beginning,
        console_printer)
    results, exitcode = run_coala(
        show_bears=partial_show_bears,
        print_results=print_results,
        acquire_settings=acquire_settings,
        print_section_beginning=partial_print_sec_beg,
        nothing_done=nothing_done)

    return exitcode



import autoflake

from coalib.bearlib.abstractions.CorrectionBasedBear import CorrectionBasedBear


class PyUnusedCodeBear(CorrectionBasedBear):
    GET_REPLACEMENT = staticmethod(
        lambda file: (autoflake.fix_code(''.join(file)).splitlines(True), []))
    RESULT_MESSAGE = "This file contains unused source code."

    def run(self, filename, file):
        """
        Detects unused code. This functionality is limited to:

        - Unneeded pass statements.
        - Unneeded builtin imports. (Others might have side effects.)
        """
        for result in self.retrieve_results(filename, file):
            yield result



import hashlib
import pickle
import os

from coalib.misc.Constants import Constants


def get_tag_path(tag, project):
    """
    Creates a hash value that is used for tagging and creates a path for it.

    :param tag:     The name for the tag.
    :param project: The related coafile.
    :return:        A path containing a hash as filename that identifies the
                    given parameters.
    """
    path = os.path.join(project, tag)
    hash = hashlib.sha224(path.encode()).hexdigest()
    return os.path.join(Constants.TAGS_DIR, hash)


def tag_results(tag, project, results):
    """
    This method takes a tag provided from the user and saves the results
    dictionary output by coala to a file. That file is uniquely identified by
    the combined hash of the project (i.e. coafile) and the provided tag.

    :param tag:     Tag provided by user.
    :param project: Path to the coafile the results belong to.
    :param results: Results dictionary generated by coala.
    """
    with open(get_tag_path(tag, project), 'wb+') as file:
        pickle.dump(results, file)


def load_tagged_results(tag, project):
    """
    Retrieves results previously stored with tag_results.

    :param tag:     The tag name.
    :param project: Path to the coafile the results belong to.
    :return:        A results dictionary, as generated by coala.
    """
    with open(get_tag_path(tag, project), 'rb') as file:
        return pickle.load(file)


def delete_tagged_results(tag, project):
    """
    Deletes previously tagged results.

    :param tag:     The tag name.
    :param project: Path to the coafile the results belong to.
    """
    file_path = get_tag_path(tag, project)
    if os.path.exists(file_path):
        os.remove(file_path)



from coalib.results.Result import Result, RESULT_SEVERITY
from coalib.bears.LocalBear import LocalBear


class KeywordBear(LocalBear):
    def run(self,
            filename,
            file,
            cs_keywords: list,
            ci_keywords: list):
        '''
        Checks the code files for given keywords.

        :param cs_keywords: A list of keywords to search for (case sensitive).
                            Usual examples are TODO and FIXME.
        :param ci_keywords: A list of keywords to search for (case
                            insensitive).
        '''
        results = list()

        for i in range(len(ci_keywords)):
            ci_keywords[i] = ci_keywords[i].lower()

        for line_number, line in enumerate(file):
            for keyword in cs_keywords:
                results += self.check_line_for_keyword(line,
                                                       filename,
                                                       line_number,
                                                       keyword)

            for keyword in ci_keywords:
                results += self.check_line_for_keyword(line.lower(),
                                                       filename,
                                                       line_number,
                                                       keyword)

        return results

    def check_line_for_keyword(self, line, filename, line_number, keyword):
        pos = line.find(keyword)
        if pos != -1:
            return [Result.from_values(
                origin=self,
                message="The line contains the keyword `{}`."
                        .format(keyword),
                file=filename,
                line=line_number+1,
                column=pos+1,
                end_line=line_number+1,
                end_column=pos+len(keyword)+1,
                severity=RESULT_SEVERITY.INFO)]

        return []



import os
import platform


class Constants:
    THIS_IS_A_BUG = ("This is a bug. We are sorry for the inconvenience. "
                     "Please contact the developers for assistance.")

    CRASH_MESSAGE = ("An unknown error occurred. This is a bug. We are "
                     "sorry for the inconvenience. Please contact the "
                     "developers for assistance. During execution of "
                     "coala an exception was raised. This should never "
                     "happen. When asked for, the following information "
                     "may help investigating:")

    OBJ_NOT_ACCESSIBLE = "{} is not accessible and will be ignored!"

    TRUE_STRINGS = ['1',
                    "on",
                    'y',
                    'yes',
                    "yeah",
                    "sure",
                    'true',
                    'definitely',
                    'yup',
                    "right"]

    FALSE_STRINGS = ['0',
                     'off',
                     'n',
                     'no',
                     'nope',
                     'nah',
                     'false',
                     "wrong"]

    # This string contains many unicode characters to challenge tests.
    COMPLEX_TEST_STRING = ("4 r34l ch4ll3n63: 123 ÄÖü ABc @€¥ §&% {[( ←↓→↑ "
                           "ĦŊħ ß°^ \\\n\u2192")

    # Results from coverage for unittests are stored here.
    COVERAGE_DIR = "./.coverageresults"

    # Path to the coalib directory
    coalib_root = os.path.join(os.path.dirname(__file__),
                               os.path.pardir)

    # Path to the directory containing the default bears
    coalib_bears_root = os.path.join(coalib_root, os.path.pardir, "bears")

    # Path to the language definition files
    language_definitions = os.path.join(coalib_root,
                                        "bearlib",
                                        "languages",
                                        "definitions")

    system_coafile = os.path.join(coalib_root, "default_coafile")

    user_coafile = os.path.join(os.path.expanduser("~"), ".coarc")

    default_coafile = ".coafile"

    VERSION_FILE = os.path.join(coalib_root, "VERSION")
    with open(VERSION_FILE, 'r') as ver:
        VERSION = ver.readline().strip()

    BUS_NAME = "org.coala_analyzer.v1"

    if platform.system() == 'Windows':  # pragma: no cover
        USER_DIR = os.path.join(os.getenv("APPDATA"), "coala")
    else:
        USER_DIR = os.path.join(os.path.expanduser("~"), ".local", "coala")

    TAGS_DIR = os.path.join(USER_DIR, "tags")
    try:
        os.makedirs(TAGS_DIR)
    except:  # pragma: no cover
        pass



from coalib.bearlib.abstractions.CorrectionBasedBear import CorrectionBasedBear


class MarkdownBear(CorrectionBasedBear):
    BINARY = 'remark'
    RESULT_MESSAGE = "The text does not comply to the set style."

    def run(self, filename, file):
        """
        Raises issues against style violations on markdown files.
        """
        cli = r'-s "bullet: \"*\", fences: true"'
        for result in self.retrieve_results(filename, file, cli_options=cli):
            yield result



import platform

from coalib.parsing.StringProcessing import escape


def escape_path_argument(path, os=platform.system()):
    """
    Makes a raw path ready for using as parameter in a shell command (escapes
    illegal characters, surrounds with quotes etc.).

    :param path: The path to make ready for shell.
    :param os:   The shell platform to escape the path argument for. Possible
                 values are "Windows" and "Linux" (others will be ignored and
                 return the given path without modification).
    :return:     The escaped path argument.
    """
    if os == "Windows":
        # If a quote (") occurs in path (which is illegal for NTFS file
        # systems, but maybe for others), escape it by preceding it with
        # a caret (^).
        return '"' + escape(path, '"', '^') + '"'
    elif os == "Linux":
        return escape(path, " ")
    else:
        # Any other non-supported system doesn't get a path escape.
        return path



from pyprint.ClosableObject import ClosableObject


class HTMLWriter(ClosableObject):
    """
    Printer for outputting HTML Log files.

    :param filename:            the name of the file to put the data into
                                (string).
    :param indentation_per_tag: spaces used to indent every subsequent HTML
                                tag.
    :raises TypeError:          if directory of given file doesn't exist or in
                                case of access problems.
    """

    def __init__(self, filename, indentation_per_tag=2, indentation=0):
        ClosableObject.__init__(self)

        self.indentation_per_tag = indentation_per_tag
        self.indentation = indentation
        self.file = None
        self.filename = filename

        if not isinstance(filename, str):
            raise TypeError("filename must be a string")

        self.file = open(filename, 'w+')
        self.__write_header()

    def _close(self):
        # Check if the file object is NoneType, trying to close a None object
        # does not make sense
        if self.file is not None:
            self.__write_footer()
            self.file.close()

    def __write_header(self):
        self.write("<!DOCTYPE html>")
        self.open_tag("html")

    def __write_footer(self):
        self.close_tag("html")

    def write_comment(self, *comments):
        """
        Function for writing HTML comments in the output HTML log files.

        :param comments: an arbitrary number of comments to add to the HTML
                         log file
        """
        for comment in comments:
            self.write("<!-- " + comment + " -->")

    def write_tag(self, tag, content="", **tagargs):
        """
        Function for writing an HTML tag, along with the required tag
        attributes and content.

        :param tag:     HTML Tag for formatting the content.
        :param content: content to output into the HTML Log file.
        :param tagargs: arbitrary HTML tag attributes mapped to their
                        respective values. Ordering of the tags is
                        not preserved.
        """
        name = tag
        for arg in tagargs:
            name += " " + arg + "=\"" + tagargs[arg] + "\""

        if content == "":
            self.write("<"+name+"/>")
            return

        self.open_tag(name)
        self.write(content)
        self.close_tag(tag)

    def open_tag(self, tag_name):
        """
        Function to open HTML tag. e.g. <p>

        :param tag_name: the name of HTML Tag to written in the output logfile.
        """
        self.write("<"+tag_name+">")
        self.indentation += 4

    def close_tag(self, tag_name):
        """
        Function to close an open HTML tag. e.g. </p>

        :param tag_name: the name of HTML Tag to be written to output logfile.
        """
        self.indentation -= 4
        self.write("</"+tag_name+">")

    def write(self, *args):
        """
        Function to write in the given output HTML log file.

        :param args: arbitrary number of arguments to be written to output
                     logfile.
        """
        for line in args:
            self.file.write(" "*self.indentation + line + "\n")
